==> ja-WiKicker-0.23.tbz: lib/perl5/site_perl/5.8.5/WiKicker/Tie/File.pm <==
#
# WiKicker
#
#  File.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: File.pm,v 1.4 2004/05/30 04:09:07 naney Exp $
#
#

package WiKicker::Tie::File;

use 5.005;
use strict;
use WiKicker::File;
use File::Path;
use Fcntl ':flock';
use Carp;

sub new {
  my ($class, $directory, $options) = @_;
  my $self = bless { directory => $directory,
                     options => $options
                   }, $class;

  $self->_initialize;

  return $self;
}

sub _initialize {
  my $self = shift;
  unless(-d $self->{directory}) {
    #mkdir($self->{directory}, 0777) || croak "Can't mkdir: $!";
    mkpath([ $self->{directory} ], 0, 0777);
  }
}

sub TIEHASH {
  my $class = shift;

  return $class->new(@_);
}

sub FETCH {
  my ($self, $key) = @_;

  $self->shared_lock_key($key);
  my $value;
  #warn "open: " . $self->file_full_name($key);
  if (open(FILE, $self->file_full_name($key))) {
    local $/;
    $value = <FILE>;
    close(FILE);
  }
  $self->unlock_key($key);

  return $value;
}

sub STORE {
  my ($self, $key, $value) = @_;

  $self->_store($key, $value, 0);
}

sub append {
  my ($self, $key, $value) = @_;

  $self->_store($key, $value, 1);
}

sub _write_file {
  my ($self, $file, $value) = @_;
  if (open(FILE, ">$file")) {
    binmode(FILE);
    print FILE $value;
    close(FILE);
    $self->_delete_index_file;
    return 1;
  }
  else {
    return 0;
  }
}

sub _append_file {
  my ($self, $file, $value) = @_;
  if (open(FILE, ">>$file")) {
    print FILE $value;
    close(FILE);
    $self->_delete_index_file;
    return 1;
  }
  else {
    return 0;
  }
}

sub _store {
  my ($self, $key, $value, $append) = @_;
  $self->exclusive_lock_key($key);
  my $file = $self->file_full_name($key);
  unless ($append) {
    if ($self->_write_file($file, $value)) {
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      croak "Can't store data to: $file";
    }
  }
  else {
    if ($self->_append_file($file, $value)) {
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      croak "Can't append data to: $file";
    }
  }
}

sub EXISTS {
  my ($self, $key) = @_;
  my $file = $self->file_full_name($key);
  if (-e $file) {
    return 1;
  }
  else {
    return 0;
  }
}

sub DELETE {
  my ($self, $key) = @_;

  $self->_delete($key);
}

sub _delete {
  my ($self, $key) = @_;

  $self->_delete_work_file($key);
}

sub _delete_work_file {
  my ($self, $key) = @_;

  my $file = $self->file_full_name($key);
  unlink($file);
  $self->_delete_index_file;
}

sub CLEAR {
  my $self = shift;

  unless ($self->{CLOBER} > 1) {
    die;
  }
  my @all_keys = $self->all_keys;
  for my $key (@all_keys) {
    $self->_delete_work_file($key);
  }
}

sub FIRSTKEY {
  my $self = shift;

  @{$self->{key_list}} = $self->all_keys;
  return shift @{$self->{key_list}};
}

sub NEXTKEY {
  my $self = shift;

  return shift @{$self->{key_list}};
}

sub all_keys {
  my $self = shift;

  my $list = $self->all_keys_ref;
  return @$list;
}

sub all_keys_ref {
  my $self = shift;

  my $keys = $self->_all_keys_ref_from_index_file;
  if (defined $keys) {
    return $keys;
  }

  $keys = $self->_all_keys_ref_from_files;
  $self->_write_index_file($keys);

  return $keys;
}


sub _all_keys_ref_from_files {
  my $self = shift;
  #warn "process $$: all keys called";

  opendir(DIR, $self->{directory}) || croak "Can't open directory: $!";
  #my @result =
  #  map { file_name_to_key($_) }
  #  grep { ! ($_ eq '.' || $_ eq '..' ||
  #            $_ eq 'RCS' || $_ =~ /-lock$/) } readdir(DIR);
  my @result;
  my $file;
  while (defined ($file = readdir(DIR))) {
    #next if $file eq '.';
    #next if $file eq '..';
    #next if $file eq 'RCS';
    next if $file eq '.'
      or $file eq '..'
        or $file eq 'RCS'
          or $file eq 'index';
    next if $file =~ /-lock$/;
    push(@result, file_name_to_key($file));
  }
  closedir(DIR);

  return \@result;
}

sub all_keys_with_predicate {
  my ($self, $predicate) = @_;

  return grep { &$predicate($_) } @{$self->all_keys_ref};
}

sub DESTROY {
  my $self = shift;
}

sub file_full_name {
  my ($self, $key) = @_;

  return $self->{directory} . '/' . key_to_file_name($key);
}

sub lock_file_name {
  my ($self, $key) = @_;
  return $self->file_full_name($key) . '-lock';
}

sub all_lock_file_name {
  my $self = shift;
  return $self->{directory} . '/' . 'alllock';
}

# class method
sub key_to_file_name {
#   my $key = shift;

#   my $name = encode_base64($key, '');
#   chomp $name;
#   $name =~ tr|/|.|;
#   return $name;
  return WiKicker::File::key_to_file_name(shift);
}

# class method
sub file_name_to_key {
  #my $file_name = shift;
  #$file_name =~ tr|.|/|;
  #return decode_base64($file_name);
  return WiKicker::File::file_name_to_key(shift);
}


# for override.
sub shared_lock_key {
}

# for override.
sub exclusive_lock_key {
}

# for override;
sub unlock_key {
}

sub _index_file_name {
  my $self = shift;

  return $self->{directory} . '/index';
}

sub _all_keys_ref_from_index_file {
  my $self = shift;

  my $index_file = $self->_index_file_name;

  open(LOCK, ">$index_file-lock") || die "Can't open lockfile $index_file->lock: $!";
  flock(LOCK, LOCK_EX);

  unless (-e $index_file) {
    flock(LOCK, LOCK_UN);
    unlink("$index_file-lock");
    return undef;
  }

  unless (open(FILE, $index_file)) {
    flock(LOCK, LOCK_UN);
    unlink("$index_file-lock");
    croak "Can't open index file $index_file: $!";
  }
  my @keys;
  while (<FILE>) {
    chomp;
    push(@keys, $_);
  }
  close(FILE);

  flock(LOCK, LOCK_UN);
  unlink("$index_file-lock");

  return \@keys;
}

sub _write_index_file {
  my ($self, $keys) = @_;

  my $index_file = $self->_index_file_name;
  open(LOCK, ">$index_file-lock") || die "Can't open lockfile $index_file->lock: $!";
  flock(LOCK, LOCK_EX);

  unless (open(FILE, ">$index_file")) {
    flock(LOCK, LOCK_UN);
    unlink("$index_file-lock");
    croak "Can't open index file $index_file: $!";
  }
  for my $key (@$keys) {
    print FILE $key, "\n";
  }
  flock(LOCK, LOCK_UN);
  unlink("$index_file-lock");
  close(FILE);
}

sub _delete_index_file {
  my $self = shift;

  my $index_file = $self->_index_file_name;
  open(LOCK, ">$index_file-lock") || die "Can't open lockfile $index_file->lock: $!";
  flock(LOCK, LOCK_EX);

  if (-e $index_file) {
    flock(LOCK, LOCK_UN);
    unlink("$index_file-lock");
    unlink($index_file) ||
      croak "Can't remove index file $index_file: $!";
  }

  flock(LOCK, LOCK_UN);
  unlink("$index_file-lock");
}

1;

==> ja-WiKicker-0.26.tbz: lib/perl5/site_perl/5.8.9/WiKicker/Tie/File.pm <==
#
# WiKicker
#
#  File.pm
#
#
# Copyright (C) 2003,2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: File.pm,v 1.8 2005/05/10 12:55:36 naney Exp $
#
#

package WiKicker::Tie::File;

use 5.005;
use strict;
use WiKicker::File;
use File::Path;
use File::Spec;
use Fcntl ':flock';
use Carp;

sub new {
  my ($class, $directory, $options) = @_;
  my $self = bless { directory => $directory,
                     options => $options
                   }, $class;

  $self->_initialize;

  return $self;
}

sub _initialize {
  my $self = shift;
  unless(-d $self->{directory}) {
    #mkdir($self->{directory}, 0777) || croak "Can't mkdir: $!";
    eval { mkpath([ $self->{directory} ], 0, 0777) };
    if ($@) {
      my $message = $@;
      $message =~ s/at\s+(.*)line\s+(\d+)//;
      croak q(Can't create directory ') . $self->{directory} . "': $message";
    }
  }
}

sub TIEHASH {
  my $class = shift;

  return $class->new(@_);
}

sub FETCH {
  my ($self, $key) = @_;

  $self->shared_lock_key($key);
  my $value;
  #warn "open: " . $self->file_full_name($key);
  if (open(FILE, $self->file_full_name($key))) {
    binmode(FILE);
    local $/;
    $value = <FILE>;
    close(FILE);
  }
  $self->unlock_key($key);

  return $value;
}

sub STORE {
  my ($self, $key, $value) = @_;

  $self->_store($key, $value, 0);
}

sub append {
  my ($self, $key, $value) = @_;

  $self->_store($key, $value, 1);
}

sub _write_file {
  my ($self, $file, $value) = @_;
  if (open(FILE, ">$file")) {
    binmode(FILE);
    print FILE $value;
    close(FILE);
    $self->_delete_index_file;
    return 1;
  }
  else {
    return 0;
  }
}

sub _append_file {
  my ($self, $file, $value) = @_;
  if (open(FILE, ">>$file")) {
    binmode(FILE);
    print FILE $value;
    close(FILE);
    $self->_delete_index_file;
    return 1;
  }
  else {
    return 0;
  }
}

sub _store {
  my ($self, $key, $value, $append) = @_;
  $self->exclusive_lock_key($key);
  my $file = $self->file_full_name($key);
  unless ($append) {
    if ($self->_write_file($file, $value)) {
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      croak "Can't store data to: $file";
    }
  }
  else {
    if ($self->_append_file($file, $value)) {
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      croak "Can't append data to: $file";
    }
  }
}

sub EXISTS {
  my ($self, $key) = @_;
  my $file = $self->file_full_name($key);
  if (-e $file) {
    return 1;
  }
  else {
    return 0;
  }
}

sub DELETE {
  my ($self, $key) = @_;

  $self->_delete($key);
}

sub _delete {
  my ($self, $key) = @_;

  $self->_delete_work_file($key);
}

sub _delete_work_file {
  my ($self, $key) = @_;

  my $file = $self->file_full_name($key);
  unlink($file);
  $self->_delete_index_file;
}

sub CLEAR {
  my $self = shift;

  unless ($self->{CLOBER} > 1) {
    die;
  }
  my @all_keys = $self->all_keys;
  for my $key (@all_keys) {
    $self->_delete_work_file($key);
  }
}

sub FIRSTKEY {
  my $self = shift;

  @{$self->{key_list}} = $self->all_keys;
  return shift @{$self->{key_list}};
}

sub NEXTKEY {
  my $self = shift;

  return shift @{$self->{key_list}};
}

sub all_keys {
  my $self = shift;

  my $list = $self->all_keys_ref;
  return @$list;
}

sub all_keys_ref {
  my $self = shift;

  my $keys = $self->_all_keys_ref_from_index_file;
  if (defined $keys) {
    return $keys;
  }

  $keys = $self->_all_keys_ref_from_files;
  $self->_write_index_file($keys);

  return $keys;
}


sub _all_keys_ref_from_files {
  my $self = shift;
  #warn "process $$: all keys called";

  opendir(DIR, $self->{directory}) || croak "Can't open directory: $!";
  #my @result =
  #  map { file_name_to_key($_) }
  #  grep { ! ($_ eq '.' || $_ eq '..' ||
  #            $_ eq 'RCS' || $_ =~ /-lock$/) } readdir(DIR);
  my @result;
  my $file;
  while (defined ($file = readdir(DIR))) {
    #next if $file eq '.';
    #next if $file eq '..';
    #next if $file eq 'RCS';
    next if $file eq '.'
      or $file eq '..'
        or $file eq 'RCS'
          or $file eq 'index';
    next if $file =~ /-lock$/;
    push(@result, file_name_to_key($file));
  }
  closedir(DIR);

  return \@result;
}

sub DESTROY {
  my $self = shift;
}

sub file_full_name {
  my ($self, $key) = @_;

  return File::Spec->catfile($self->{directory}, key_to_file_name($key));
}

sub lock_file_name {
  my ($self, $key) = @_;
  return $self->file_full_name($key) . '-lock';
}

sub all_lock_file_name {
  my $self = shift;
  return File::Spec->catfile($self->{directory}, 'alllock');
}

# class method
sub key_to_file_name {
#   my $key = shift;

#   my $name = encode_base64($key, '');
#   chomp $name;
#   $name =~ tr|/|.|;
#   return $name;
  return WiKicker::File::key_to_file_name(shift);
}

# class method
sub file_name_to_key {
  #my $file_name = shift;
  #$file_name =~ tr|.|/|;
  #return decode_base64($file_name);
  return WiKicker::File::file_name_to_key(shift);
}


# for override.
sub shared_lock_key {
}

# for override.
sub exclusive_lock_key {
}

# for override;
sub unlock_key {
}

sub _index_file_name {
  my $self = shift;

  return File::Spec->catfile($self->{directory}, 'index');
}

sub _all_keys_ref_from_index_file {
  my $self = shift;

  my $index_file = $self->_index_file_name;

  open(LOCK, ">$index_file-lock") || die "Can't open lockfile $index_file->lock: $!";
  flock(LOCK, LOCK_EX) || die "Can't flock: $!";

  unless (-e $index_file) {
    $self->_unlink_index_lock_file;
    flock(LOCK, LOCK_UN);
    return undef;
  }

  unless (open(FILE, $index_file)) {
    $self->_unlink_index_lock_file;
    flock(LOCK, LOCK_UN);
    croak "Can't open index file $index_file: $!";
  }
  my @keys;
  while (<FILE>) {
    chomp;
    push(@keys, $_);
  }
  close(FILE);

  $self->_unlink_index_lock_file;
  flock(LOCK, LOCK_UN);

  return \@keys;
}

sub _write_index_file {
  my ($self, $keys) = @_;

  my $index_file = $self->_index_file_name;
  open(LOCK, ">$index_file-lock") || die "Can't open lockfile $index_file->lock: $!";
  flock(LOCK, LOCK_EX)  || die "Can't flock: $!";

  unless (open(FILE, ">$index_file")) {
    $self->_unlink_index_lock_file;
    flock(LOCK, LOCK_UN);
    croak "Can't open index file $index_file: $!";
  }
  for my $key (@$keys) {
    print FILE $key, "\n";
  }
  close(FILE);
  $self->_unlink_index_lock_file;
  flock(LOCK, LOCK_UN);
}

sub _delete_index_file {
  my $self = shift;

  my $index_file = $self->_index_file_name;
  open(LOCK, ">$index_file-lock") || die "Can't open lockfile $index_file->lock: $!";
  flock(LOCK, LOCK_EX) || die "Can't flock: $!";

  if (-e $index_file) {
    unlink($index_file) ||
      croak "Can't remove index file $index_file: $!";
  }

  $self->_unlink_index_lock_file;
  flock(LOCK, LOCK_UN);
}

sub _unlink_index_lock_file {
  my $self = shift;

  my $index_file = $self->_index_file_name;
  unless ($^O =~ /Win32/) {
    unlink("$index_file-lock") || warn "Can't unlink $index_file-lock: $!";
  }
}

1;
